from http import HTTPStatus
from typing import TYPE_CHECKING

from starlette.testclient import TestClient

from testing.mock import MockSettings
from yak_server.helpers.settings import CompetitionSettings, get_settings

if TYPE_CHECKING:
    from fastapi import FastAPI


def test_competition(app_with_valid_jwt_config: "FastAPI") -> None:
    app_with_valid_jwt_config.dependency_overrides[get_settings] = MockSettings(
        data_folder_relative="test_competition_v1",
        competition="WORLD_CUP_2026",
        competition_settings=CompetitionSettings(
            description_fr="Coupe du monde 2026",
            description_en="World Cup 2026",
        ),
    )

    client = TestClient(app_with_valid_jwt_config)

    # Test with default language (fr)
    response_default = client.get("/api/v1/competition")

    assert response_default.status_code == HTTPStatus.OK
    assert response_default.json() == {
        "ok": True,
        "result": {
            "code": "WORLD_CUP_2026",
            "description": "Coupe du monde 2026",
            "logo": {"url": "/api/v1/competition/logo"},
        },
    }

    # Test with French language explicitly
    response_fr = client.get("/api/v1/competition?lang=fr")

    assert response_fr.status_code == HTTPStatus.OK
    assert response_fr.json()["result"]["description"] == "Coupe du monde 2026"

    # Test with English language
    response_en = client.get("/api/v1/competition?lang=en")

    assert response_en.status_code == HTTPStatus.OK
    assert response_en.json()["result"]["description"] == "World Cup 2026"

    # Test with unsupported language
    response_invalid_lang = client.get("/api/v1/competition?lang=de")

    assert response_invalid_lang.status_code == HTTPStatus.UNPROCESSABLE_ENTITY
    assert response_invalid_lang.json() == {
        "ok": False,
        "error_code": HTTPStatus.UNPROCESSABLE_ENTITY,
        "description": [{"field": "query -> lang", "error": "Input should be 'fr' or 'en'"}],
    }


def test_competition_logo(app_with_valid_jwt_config: "FastAPI") -> None:
    app_with_valid_jwt_config.dependency_overrides[get_settings] = MockSettings(
        data_folder_relative="test_competition_v1",
        competition="WORLD_CUP_2026",
        competition_settings=CompetitionSettings(
            description_fr="Coupe du monde 2026",
            description_en="World Cup 2026",
        ),
    )

    client = TestClient(app_with_valid_jwt_config)

    response_logo = client.get("/api/v1/competition/logo")

    assert response_logo.status_code == HTTPStatus.OK
    assert "svg" in response_logo.headers["content-type"]



from typing import TYPE_CHECKING

from pydantic import UUID4, BaseModel, ConfigDict, PositiveInt

from yak_server.helpers.language import Lang, get_language_description

from .groups import GroupIn, GroupOut
from .phases import PhaseOut
from .teams import FlagOut, TeamIn, TeamModifyBinaryBetIn, TeamWithWonOut

if TYPE_CHECKING:
    from yak_server.database.models import BinaryBetModel


class BinaryBetIn(BaseModel):
    is_one_won: bool | None = None
    index: PositiveInt
    team1: TeamIn
    team2: TeamIn
    group: GroupIn


class BinaryBetOut(BaseModel):
    id: UUID4
    locked: bool
    team1: TeamWithWonOut | None = None
    team2: TeamWithWonOut | None = None

    @classmethod
    def from_instance(
        cls,
        binary_bet: "BinaryBetModel",
        *,
        locked: bool,
        lang: "Lang",
    ) -> "BinaryBetOut":
        return cls(
            id=binary_bet.id,
            locked=locked,
            team1=(
                TeamWithWonOut(
                    id=binary_bet.match.team1.id,
                    code=binary_bet.match.team1.code,
                    description=get_language_description(binary_bet.match.team1, lang),
                    won=binary_bet.bet_from_is_one_won()[0],
                    flag=FlagOut(url=binary_bet.match.team1.internal_flag_url),
                )
                if binary_bet.match.team1
                else None
            ),
            team2=(
                TeamWithWonOut(
                    id=binary_bet.match.team2.id,
                    code=binary_bet.match.team2.code,
                    description=get_language_description(binary_bet.match.team2, lang),
                    won=binary_bet.bet_from_is_one_won()[1],
                    flag=FlagOut(url=binary_bet.match.team2.internal_flag_url),
                )
                if binary_bet.match.team2
                else None
            ),
        )


class Group(BaseModel):
    id: UUID4


class BinaryBetWithGroupIdOut(BaseModel):
    id: UUID4
    locked: bool
    group: Group
    team1: TeamWithWonOut | None = None
    team2: TeamWithWonOut | None = None

    @classmethod
    def from_instance(
        cls,
        binary_bet: "BinaryBetModel",
        *,
        locked: bool,
        lang: Lang,
    ) -> "BinaryBetWithGroupIdOut":
        return cls(
            id=binary_bet.id,
            locked=locked,
            group=Group(id=binary_bet.match.group_id),
            team1=(
                TeamWithWonOut(
                    id=binary_bet.match.team1.id,
                    code=binary_bet.match.team1.code,
                    description=get_language_description(binary_bet.match.team1, lang),
                    won=binary_bet.bet_from_is_one_won()[0],
                    flag=FlagOut(url=binary_bet.match.team1.internal_flag_url),
                )
                if binary_bet.match.team1
                else None
            ),
            team2=(
                TeamWithWonOut(
                    id=binary_bet.match.team2.id,
                    code=binary_bet.match.team2.code,
                    description=get_language_description(binary_bet.match.team2, lang),
                    won=binary_bet.bet_from_is_one_won()[1],
                    flag=FlagOut(url=binary_bet.match.team2.internal_flag_url),
                )
                if binary_bet.match.team2
                else None
            ),
        )


class BinaryBetResponse(BaseModel):
    phase: PhaseOut
    group: GroupOut
    binary_bet: BinaryBetOut


class ModifyBinaryBetIn(BaseModel):
    is_one_won: bool | None = None
    team1: TeamModifyBinaryBetIn | None = None
    team2: TeamModifyBinaryBetIn | None = None

    model_config = ConfigDict(extra="forbid")
